#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Tests for manipulating pci device via the DB API"""

from zun.common import context
from zun.common import exception
import zun.conf
from zun.db import api as dbapi
from zun.objects import fields as z_fields
from zun.tests.unit.db import base
from zun.tests import uuidsentinel

CONF = zun.conf.CONF


class DbPciDeviceTestCase(base.DbTestCase, base.ModelsObjectComparatorMixin):

    def setUp(self):
        super(DbPciDeviceTestCase, self).setUp()
        self.admin_context = context.get_admin_context()
        self._compute_node = None
        self.ignored_keys = ['id', 'updated_at', 'created_at']

    def _get_fake_pci_devs(self):
        v1 = {'id': 3353,
              'uuid': uuidsentinel.pci_device1,
              'compute_node_uuid': uuidsentinel.compute_node,
              'address': '0000:0f:08.7',
              'vendor_id': '8086',
              'product_id': '1520',
              'numa_node': 1,
              'dev_type': z_fields.PciDeviceType.SRIOV_VF,
              'dev_id': 'pci_0000:0f:08.7',
              'extra_info': '{}',
              'label': 'label_8086_1520',
              'status': z_fields.PciDeviceStatus.AVAILABLE,
              'container_uuid': '00000000-0000-0000-0000-000000000010',
              'request_id': None,
              'parent_addr': '0000:0f:00.1'}
        v2 = {'id': 3356,
              'uuid': uuidsentinel.pci_device3356,
              'compute_node_uuid': uuidsentinel.compute_node,
              'address': '0000:0f:03.7',
              'parent_addr': '0000:0f:03.0',
              'vendor_id': '8083',
              'product_id': '1523',
              'numa_node': 0,
              'dev_type': z_fields.PciDeviceType.SRIOV_VF,
              'dev_id': 'pci_0000:0f:08.7',
              'extra_info': '{}',
              'label': 'label_8086_1520',
              'status': z_fields.PciDeviceStatus.AVAILABLE,
              'container_uuid': '00000000-0000-0000-0000-000000000010',
              'request_id': None}
        return v1, v2

    @property
    def compute_node(self):
        if self._compute_node is None:
            self._compute_node = dbapi.create_compute_node(
                self.admin_context,
                {'uuid': uuidsentinel.compute_node,
                 'rp_uuid': uuidsentinel.compute_node,
                 'hostname': 'fake_compute_node',
                 'mem_total': 40960,
                 'mem_free': 20480,
                 'mem_available': 20480,
                 'mem_used': 20480,
                 'total_containers': 0,
                 'running_containers': 0,
                 'paused_containers': 0,
                 'stopped_containers': 0,
                 'cpus': 48,
                 'cpu_used': 24})
        return self._compute_node

    def _create_fake_pci_devs(self):
        v1, v2 = self._get_fake_pci_devs()
        for i in v1, v2:
            i['compute_node_uuid'] = self.compute_node['uuid']

        dbapi.update_pci_device(v1['compute_node_uuid'],
                                v1['address'], v1)
        dbapi.update_pci_device(v2['compute_node_uuid'],
                                v2['address'], v2)

        return (v1, v2)

    def test_get_pci_device_by_addr(self):
        v1, v2 = self._create_fake_pci_devs()
        result = dbapi.get_pci_device_by_addr(self.compute_node['uuid'],
                                              '0000:0f:08.7')
        self._assertEqualObjects(v1, result, self.ignored_keys)

    def test_get_pci_device_by_addr_not_found(self):
        self._create_fake_pci_devs()
        self.assertRaises(exception.PciDeviceNotFound,
                          dbapi.get_pci_device_by_addr,
                          uuidsentinel.compute_node, '0000:0f:08:09')

    def test_get_all_pci_device_by_parent_addr(self):
        v1, v2 = self._create_fake_pci_devs()
        results = dbapi.get_all_pci_device_by_parent_addr(
            uuidsentinel.compute_node, '0000:0f:00.1')
        self._assertEqualListsOfObjects([v1], results, self.ignored_keys)

    def test_get_all_pci_device_by_parent_addr_empty(self):
        v1, v2 = self._create_fake_pci_devs()
        results = dbapi.get_all_pci_device_by_parent_addr(
            uuidsentinel.compute_node, '0000:0f:01.6')
        self.assertEqual(len(results), 0)

    def test_get_pci_device_by_id(self):
        v1, v2 = self._create_fake_pci_devs()
        result = dbapi.get_pci_device_by_id(3353)
        self._assertEqualObjects(v1, result, self.ignored_keys)

    def test_get_pci_device_by_id_not_found(self):
        self._create_fake_pci_devs()
        self.assertRaises(exception.PciDeviceNotFoundById,
                          dbapi.get_pci_device_by_id, 3354)

    def test_get_all_pci_device_by_node(self):
        v1, v2 = self._create_fake_pci_devs()
        results = dbapi.get_all_pci_device_by_node(uuidsentinel.compute_node)
        self._assertEqualListsOfObjects(results, [v1, v2], self.ignored_keys)

    def test_get_all_pci_device_by_node_empty(self):
        v1, v2 = self._get_fake_pci_devs()
        results = dbapi.get_all_pci_device_by_node(9)
        self.assertEqual(len(results), 0)

    def test_get_pci_device_by_container_uuid(self):
        v1, v2 = self._create_fake_pci_devs()
        v1['status'] = z_fields.PciDeviceStatus.ALLOCATED
        v2['status'] = z_fields.PciDeviceStatus.ALLOCATED
        dbapi.update_pci_device(v1['compute_node_uuid'],
                                v1['address'], v1)
        dbapi.update_pci_device(v2['compute_node_uuid'],
                                v2['address'], v2)
        results = dbapi.get_all_pci_device_by_container_uuid(
            '00000000-0000-0000-0000-000000000010')
        self._assertEqualListsOfObjects(results, [v1, v2], self.ignored_keys)

    def test_get_pci_device_by_container_uuid_check_status(self):
        v1, v2 = self._create_fake_pci_devs()
        v1['status'] = z_fields.PciDeviceStatus.ALLOCATED
        v2['status'] = z_fields.PciDeviceStatus.CLAIMED
        dbapi.update_pci_device(v1['compute_node_uuid'],
                                v1['address'], v1)
        dbapi.update_pci_device(v2['compute_node_uuid'],
                                v2['address'], v2)
        results = dbapi.get_all_pci_device_by_container_uuid(
            '00000000-0000-0000-0000-000000000010')
        self._assertEqualListsOfObjects(results, [v1], self.ignored_keys)

    def test_update_pci_device(self):
        v1, v2 = self._create_fake_pci_devs()
        v1['status'] = z_fields.PciDeviceStatus.ALLOCATED
        dbapi.update_pci_device(v1['compute_node_uuid'],
                                v1['address'], v1)
        result = dbapi.get_pci_device_by_addr(uuidsentinel.compute_node,
                                              '0000:0f:08.7')
        self._assertEqualObjects(v1, result, self.ignored_keys)

        v1['status'] = z_fields.PciDeviceStatus.CLAIMED
        dbapi.update_pci_device(v1['compute_node_uuid'],
                                v1['address'], v1)
        result = dbapi.get_pci_device_by_addr(uuidsentinel.compute_node,
                                              '0000:0f:08.7')
        self._assertEqualObjects(v1, result, self.ignored_keys)

    def test_destroy_pci_device(self):
        v1, v2 = self._create_fake_pci_devs()
        results = dbapi.get_all_pci_device_by_node(self.compute_node['uuid'])
        self._assertEqualListsOfObjects(results, [v1, v2], self.ignored_keys)
        dbapi.destroy_pci_device(v1['compute_node_uuid'], v1['address'])
        results = dbapi.get_all_pci_device_by_node(self.compute_node['uuid'])
        self._assertEqualListsOfObjects(results, [v2], self.ignored_keys)

    def test_destroy_pci_device_exception(self):
        v1, v2 = self._get_fake_pci_devs()
        self.assertRaises(exception.PciDeviceNotFound,
                          dbapi.destroy_pci_device,
                          v1['compute_node_uuid'],
                          v1['address'])



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging

import zun.conf
from zun.scheduler import filters


LOG = logging.getLogger(__name__)
CONF = zun.conf.CONF


class AvailabilityZoneFilter(filters.BaseHostFilter):
    """Filters Hosts by availability zone."""

    # Availability zones do not change within a request
    run_filter_once_per_request = True

    def host_passes(self, host_state, container, extra_spec):
        availability_zone = extra_spec.get('availability_zone') or \
            CONF.default_schedule_zone
        if not availability_zone:
            return True

        host_az = host_state.service.availability_zone
        if not host_az:
            host_az = CONF.default_availability_zone

        hosts_passes = availability_zone == host_az
        if not hosts_passes:
            LOG.debug("Availability Zone '%(az)s' requested. "
                      "%(host_state)s has AZs: %(host_az)s",
                      {'host_state': host_state,
                       'az': availability_zone,
                       'host_az': host_az})

        return hosts_passes



# Copyright 2017 Arm Limited.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""change the properties of meta_labels

Revision ID: fc27c7415d9c
Revises: bcd6410d645e
Create Date: 2017-09-07 10:56:07.489031

"""

# revision identifiers, used by Alembic.
revision = 'fc27c7415d9c'
down_revision = 'bcd6410d645e'
branch_labels = None
depends_on = None

from alembic import op

import zun


def upgrade():
    op.alter_column('capsule', 'meta_labels',
                    type_=zun.db.sqlalchemy.models.JSONEncodedDict()
                    )



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""add_exposed_ports_to_container

Revision ID: 02134de8e7d3
Revises: a019998b09b5
Create Date: 2018-08-19 19:29:51.636559

"""

# revision identifiers, used by Alembic.
revision = '02134de8e7d3'
down_revision = 'a019998b09b5'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa

import zun


def upgrade():
    op.add_column('container',
                  sa.Column('exposed_ports',
                            zun.db.sqlalchemy.models.JSONEncodedDict(),
                            nullable=True))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""upgrade

Revision ID: 71f8b4cf1dbf
Revises: 10c9668a816d
Create Date: 2018-01-29 09:09:32.297389

"""

# revision identifiers, used by Alembic.
revision = '71f8b4cf1dbf'
down_revision = '10c9668a816d'
branch_labels = None
depends_on = None

from alembic import op
from sqlalchemy.dialects import mysql


def upgrade():
    with op.batch_alter_table('capsule', schema=None) as batch_op:
        batch_op.create_unique_constraint('uniq_capsule0uuid', ['uuid'])
        batch_op.drop_column('message')

    with op.batch_alter_table('container_actions', schema=None) as batch_op:
        batch_op.create_foreign_key(
            None, 'container', ['container_uuid'], ['uuid'])

    with op.batch_alter_table('pci_device', schema=None) as batch_op:
        batch_op.create_foreign_key(
            None, 'compute_node', ['compute_node_uuid'], ['uuid'])

    with op.batch_alter_table('volume_mapping', schema=None) as batch_op:
        batch_op.alter_column('container_uuid',
                              existing_type=mysql.VARCHAR(length=36),
                              nullable=True)
        batch_op.create_foreign_key(
            None, 'container', ['container_uuid'], ['uuid'])



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import itertools

from zun.api.controllers import link


_basic_keys = (
    'availability_zone',
    )


def format_a_zone(url, a_zone):
    def transform(key, value):
        if key not in _basic_keys:
            return
        if key == 'id':
            yield ('id', value)
            yield ('links', [link.make_link(
                'self', url, 'availability_zones', value),
                link.make_link(
                    'bookmark', url,
                    'availability_zones', value,
                    bookmark=True)])
        else:
            yield (key, value)

    return dict(
        itertools.chain.from_iterable(
            transform(k, v)for k, v in a_zone.as_dict().items()))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

from neutron_lib import constants as neutron_constants
from oslo_log import log

from zun.cni.binding import base as b_base
from zun.cni import utils as cni_utils
from zun.common import privileged
from zun.network import linux_net as net_utils


LOG = log.getLogger(__name__)


@privileged.cni.entrypoint
def base_bridge_connect(vif_dict, ifname, netns, container_id, **kwargs):
    host_ifname = vif_dict['vif_name']
    mtu = vif_dict['network']['mtu']
    address = vif_dict['address']
    bridge_name = vif_dict['bridge_name']

    with b_base.get_ipdb() as h_ipdb:
        if host_ifname in h_ipdb.interfaces:
            # NOTE(dulek): This most likely means that we already run
            #              connect for this iface and there's a leftover
            #              host-side vif. Let's remove it, its peer should
            #              get deleted automatically by the kernel.
            LOG.debug('Found leftover host vif %s. Removing it before '
                      'connecting.', host_ifname)
            with h_ipdb.interfaces[host_ifname] as h_iface:
                h_iface.remove()

    if mtu:
        interface_mtu = mtu
    else:
        LOG.info("Default mtu %(mtu)s is used for interface, "
                 "for mtu of network if configured with 0",
                 {"mtu": neutron_constants.DEFAULT_NETWORK_MTU})
        interface_mtu = neutron_constants.DEFAULT_NETWORK_MTU

    with b_base.get_ipdb(netns) as c_ipdb:
        with c_ipdb.create(ifname=ifname, peer=host_ifname,
                           kind='veth') as c_iface:
            c_iface.mtu = interface_mtu
            c_iface.address = address
            c_iface.up()

        if netns:
            with c_ipdb.interfaces[host_ifname] as h_iface:
                h_iface.net_ns_pid = os.getpid()

    with b_base.get_ipdb() as h_ipdb:
        with h_ipdb.interfaces[host_ifname] as h_iface:
            h_iface.mtu = interface_mtu
            h_iface.up()

    if kwargs.get('driver') == 'bridge':
        with b_base.get_ipdb() as h_ipdb:
            with h_ipdb.interfaces[bridge_name] as h_br:
                h_br.add_port(host_ifname)


class BaseBridgeDriver(b_base.BaseBindingDriver):

    def connect(self, vif, ifname, netns, container_id, **kwargs):
        vif_dict = cni_utils.osvif_vif_to_dict(vif)
        base_bridge_connect(vif_dict, ifname, netns, container_id, **kwargs)

    def disconnect(self, vif, ifname, netns, container_id):
        pass


class BridgeDriver(BaseBridgeDriver):

    def connect(self, vif, ifname, netns, container_id):
        super(BridgeDriver, self).connect(vif, ifname, netns, container_id,
                                          driver='bridge')

    def disconnect(self, vif, ifname, netns, container_id):
        # NOTE(ivc): veth pair is destroyed automatically along with the
        # container namespace
        pass


class VIFOpenVSwitchDriver(BaseBridgeDriver):

    def connect(self, vif, ifname, netns, container_id):
        super(VIFOpenVSwitchDriver, self).connect(vif, ifname, netns,
                                                  container_id)
        # FIXME(irenab) use container_uuid (neutron port device_id)
        instance_id = 'zun'
        net_utils.create_ovs_vif_port(vif.bridge_name, vif.vif_name,
                                      vif.port_profile.interface_id,
                                      vif.address, instance_id)

    def disconnect(self, vif, ifname, netns, container_id):
        super(VIFOpenVSwitchDriver, self).disconnect(vif, ifname, netns,
                                                     container_id)
        net_utils.delete_ovs_vif_port(vif.bridge_name, vif.vif_name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os
import time

from keystoneauth1 import identity
from keystoneauth1 import session as ks
import os_client_config
from zunclient import client


ZUN_API_VERSION = '1.30'


def get_zun_client_from_env():
    # We should catch KeyError exception with the purpose of
    # source or configure openrc file.
    auth_url = os.environ['OS_AUTH_URL']
    username = os.environ['OS_USERNAME']
    password = os.environ['OS_PASSWORD']
    project_name = os.environ['OS_PROJECT_NAME']

    # Either project(user)_domain_name or project(user)_domain_id
    # would be acceptable.
    project_domain_name = os.environ.get("OS_PROJECT_DOMAIN_NAME")
    project_domain_id = os.environ.get("OS_PROJECT_DOMAIN_ID")
    user_domain_name = os.environ.get("OS_USER_DOMAIN_NAME")
    user_domain_id = os.environ.get("OS_USER_DOMAIN_ID")

    auth = identity.Password(auth_url=auth_url,
                             username=username,
                             password=password,
                             project_name=project_name,
                             project_domain_id=project_domain_id,
                             project_domain_name=project_domain_name,
                             user_domain_id=user_domain_id,
                             user_domain_name=user_domain_name)
    session = ks.Session(auth=auth)
    return client.Client(ZUN_API_VERSION, session=session)


def _get_cloud_config_auth_data(cloud='devstack-admin'):
    """Retrieves Keystone auth data to run tests

    Credentials are either read via os-client-config from the environment
    or from a config file ('clouds.yaml'). Environment variables override
    those from the config file.
    devstack produces a clouds.yaml with two named clouds - one named
    'devstack' which has user privs and one named 'devstack-admin' which
    has admin privs. This function will default to getting the devstack-admin
    cloud as that is the current expected behavior.
    """
    cloud_config = os_client_config.OpenStackConfig().get_one_cloud(cloud)
    return cloud_config.get_auth(), cloud_config.get_session()


def get_zun_client_from_creds():
    auth_plugin, session = _get_cloud_config_auth_data()
    return client.Client(ZUN_API_VERSION, session=session, auth=auth_plugin)


def wait_for_condition(condition, interval=2, timeout=60):
    start_time = time.time()
    end_time = time.time() + timeout
    while time.time() < end_time:
        result = condition()
        if result:
            return result
        time.sleep(interval)
    raise Exception(("Timed out after %s seconds.  Started on %s and ended "
                     "on %s") % (timeout, start_time, end_time))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging
from oslo_versionedobjects import fields

from zun.db import api as dbapi
from zun.objects import base


LOG = logging.getLogger(__name__)


@base.ZunObjectRegistry.register
class ExecInstance(base.ZunPersistentObject, base.ZunObject):
    # Version 1.0: Initial version
    VERSION = '1.0'

    fields = {
        'id': fields.IntegerField(),
        'container_id': fields.IntegerField(nullable=False),
        'exec_id': fields.StringField(nullable=False),
        'token': fields.StringField(nullable=True),
        'url': fields.StringField(nullable=True),
    }

    @staticmethod
    def _from_db_object(exec_inst, db_exec_inst):
        """Converts a database entity to a formal object."""
        for field in exec_inst.fields:
            setattr(exec_inst, field, db_exec_inst[field])

        exec_inst.obj_reset_changes()
        return exec_inst

    @staticmethod
    def _from_db_object_list(db_objects, cls, context):
        """Converts a list of database entities to a list of formal objects."""
        return [ExecInstance._from_db_object(cls(context), obj)
                for obj in db_objects]

    @base.remotable_classmethod
    def list_by_container_id(cls, context, container_id):
        db_objects = dbapi.list_exec_instances(
            context, filters={'container_id': container_id})
        return ExecInstance._from_db_object_list(db_objects, cls, context)

    @base.remotable
    def create(self, context):
        values = self.obj_get_changes()
        db_exec_inst = dbapi.create_exec_instance(context, values)
        self._from_db_object(self, db_exec_inst)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""add disk_quota_supported to compute_node

Revision ID: 238f94009eab
Revises: 372433c0afd2
Create Date: 2018-04-01 18:46:29.977789

"""

# revision identifiers, used by Alembic.
revision = '238f94009eab'
down_revision = '372433c0afd2'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('compute_node',
                  sa.Column('disk_quota_supported', sa.Boolean(),
                            nullable=False, default=sa.sql.false(),
                            server_default=sa.sql.false()))



# Copyright 2018 National Engineering Laboratory of electronic
# commerce and electronic payment.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from zun.common import context
import zun.conf
from zun import objects
from zun.scheduler.filters import cpuset_filter
from zun.tests import base
from zun.tests.unit.scheduler import fakes

CONF = zun.conf.CONF


class TestCpuSetFilter(base.TestCase):
    def setUp(self):
        super(TestCpuSetFilter, self).setUp()
        self.context = context.RequestContext('fake_user', 'fake_project')

    def test_cpuset_filter_pass_dedicated(self):
        self.filt_cls = cpuset_filter.CpuSetFilter()
        container = objects.Container(self.context)
        container.cpu_policy = 'dedicated'
        container.cpu = 2.0
        container.memory = '1024'
        host = fakes.FakeHostState('testhost')
        host.cpus = 6
        host.cpu_used = 0.0
        host.numa_topology = objects.NUMATopology(nodes=[
            objects.NUMANode(id=0, cpuset=set([1, 2, 3]), pinned_cpus=set([]),
                             mem_total=32739, mem_available=32739),
            objects.NUMANode(id=1, cpuset=set([4, 5, 6]), pinned_cpus=set([]),
                             mem_total=32739, mem_available=32739)]
        )
        host.enable_cpu_pinning = True
        extra_spec = {}
        self.assertTrue(self.filt_cls.host_passes(host, container, extra_spec))

    def test_cpuset_filter_fail_dedicated_1(self):
        self.filt_cls = cpuset_filter.CpuSetFilter()
        container = objects.Container(self.context)
        container.cpu_policy = 'dedicated'
        container.cpu = 4.0
        container.memory = '1024'
        host = fakes.FakeHostState('testhost')
        host.cpus = 6
        host.cpu_used = 0.0
        host.numa_topology = objects.NUMATopology(nodes=[
            objects.NUMANode(id=0, cpuset=set([1, 2, 3]), pinned_cpus=set([]),
                             mem_total=32739, mem_available=32739),
            objects.NUMANode(id=1, cpuset=set([4, 5, 6]), pinned_cpus=set([]),
                             mem_total=32739, mem_available=32739)]
        )
        host.enable_cpu_pinning = True
        extra_spec = {}
        self.assertFalse(self.filt_cls.host_passes(host,
                                                   container, extra_spec))

    def test_cpuset_filter_fail_dedicated_2(self):
        self.filt_cls = cpuset_filter.CpuSetFilter()
        container = objects.Container(self.context)
        container.cpu_policy = 'dedicated'
        container.cpu = 2.0
        container.memory = '1024'
        host = fakes.FakeHostState('testhost')
        host.cpus = 6
        host.cpu_used = 0.0
        host.numa_topology = objects.NUMATopology(nodes=[
            objects.NUMANode(id=0, cpuset=set([1, 2, 3]), pinned_cpus=set([]),
                             mem_total=32739, mem_available=32739),
            objects.NUMANode(id=1, cpuset=set([4, 5, 6]), pinned_cpus=set([]),
                             mem_total=32739, mem_available=32739)]
        )
        host.enable_cpu_pinning = False
        extra_spec = {}
        self.assertFalse(self.filt_cls.host_passes(host,
                                                   container, extra_spec))



# Copyright 2018 Arm Limited.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


"""add volumes info and addresses to capsule

Revision ID: 10c9668a816d
Revises: 8b0082d9e7c1
Create Date: 2018-01-19 15:40:35.242920

"""

# revision identifiers, used by Alembic.
revision = '10c9668a816d'
down_revision = '8b0082d9e7c1'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa

from zun.db.sqlalchemy import models


def upgrade():
    op.add_column('capsule', sa.Column('volumes_info',
                                       models.JSONEncodedDict(),
                                       nullable=True))

    op.add_column('capsule',
                  sa.Column('addresses', models.JSONEncodedDict(),
                            nullable=True))



# Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.


"""Insert status_reason to Container table

Revision ID: c5565cbaa3de
Revises: 72c6947c6636
Create Date: 2016-10-28 06:51:12.146721

"""

# revision identifiers, used by Alembic.
revision = 'c5565cbaa3de'
down_revision = '72c6947c6636'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('container',
                  sa.Column('status_reason', sa.Text(),
                            nullable=True))
